@@ -14,27 +14,30 @@ import Control.Monad.ST
 import Data.Int
 import Data.Word
 
+import qualified Data.ByteString as B
+
 import Data.Vector (Vector)
 import qualified Data.Vector as V
 
 import Data.Vector.Generic.Mutable (MVector)
 import qualified Data.Vector.Generic.Mutable as MV
 
-import qualified Data.Vector.Algorithms.Insertion as INS
-import qualified Data.Vector.Algorithms.Intro     as INT
-import qualified Data.Vector.Algorithms.Merge     as M
-import qualified Data.Vector.Algorithms.Radix     as R
-import qualified Data.Vector.Algorithms.TriHeap   as TH
-import qualified Data.Vector.Algorithms.Optimal   as O
+import qualified Data.Vector.Algorithms.Insertion    as INS
+import qualified Data.Vector.Algorithms.Intro        as INT
+import qualified Data.Vector.Algorithms.Merge        as M
+import qualified Data.Vector.Algorithms.Radix        as R
+import qualified Data.Vector.Algorithms.Heap         as H
+import qualified Data.Vector.Algorithms.Optimal      as O
+import qualified Data.Vector.Algorithms.AmericanFlag as AF
 
-import qualified Data.Vector.Algorithms.Search    as SR
+import qualified Data.Vector.Algorithms.Search       as SR
 
 type Algo      e r = forall s mv. MVector mv e => mv s e -> ST s r
 type SizeAlgo  e r = forall s mv. MVector mv e => mv s e -> Int -> ST s r
 type BoundAlgo e r = forall s mv. MVector mv e => mv s e -> Int -> Int -> ST s r
 
 args = stdArgs
-       { maxSuccess = 300
+       { maxSuccess = 1000
        , maxDiscard = 200
        }
 
@@ -45,7 +48,7 @@ check_Int_sort = forM_ algos $ \(name,algo) ->
  algos = [ ("introsort", INT.sort)
          , ("insertion sort", INS.sort)
          , ("merge sort", M.sort)
-         , ("tri-heapsort", TH.sort)
+         , ("heapsort", H.sort)
          ]
 
 check_Int_partialsort = forM_ algos $ \(name,algo) ->
@@ -53,7 +56,7 @@ check_Int_partialsort = forM_ algos $ \(name,algo) ->
  where
  algos :: [(String, SizeAlgo Int ())]
  algos = [ ("intro-partialsort", INT.partialSort)
-         , ("tri-heap partialsort", TH.partialSort)
+         , ("heap partialsort", H.partialSort)
          ]
 
 check_Int_select = forM_ algos $ \(name,algo) ->
@@ -61,21 +64,33 @@ check_Int_select = forM_ algos $ \(name,algo) ->
  where
  algos :: [(String, SizeAlgo Int ())]
  algos = [ ("intro-select", INT.select)
-         , ("tri-heap select", TH.select)
+         , ("heap select", H.select)
          ]
 
 check_radix_sorts = do
-  qc (label "Word8"       . prop_fullsort (R.sort :: Algo Word8  ()))
-  qc (label "Word16"      . prop_fullsort (R.sort :: Algo Word16 ()))
-  qc (label "Word32"      . prop_fullsort (R.sort :: Algo Word32 ()))
-  qc (label "Word64"      . prop_fullsort (R.sort :: Algo Word64 ()))
-  qc (label "Word"        . prop_fullsort (R.sort :: Algo Word   ()))
-  qc (label "Int8"        . prop_fullsort (R.sort :: Algo Int8   ()))
-  qc (label "Int16"       . prop_fullsort (R.sort :: Algo Int16  ()))
-  qc (label "Int32"       . prop_fullsort (R.sort :: Algo Int32  ()))
-  qc (label "Int64"       . prop_fullsort (R.sort :: Algo Int64  ()))
-  qc (label "Int"         . prop_fullsort (R.sort :: Algo Int    ()))
-  qc (label "(Int, Int)"  . prop_fullsort (R.sort :: Algo (Int, Int) ()))
+  qc (label "radix Word8"       . prop_fullsort (R.sort :: Algo Word8  ()))
+  qc (label "radix Word16"      . prop_fullsort (R.sort :: Algo Word16 ()))
+  qc (label "radix Word32"      . prop_fullsort (R.sort :: Algo Word32 ()))
+  qc (label "radix Word64"      . prop_fullsort (R.sort :: Algo Word64 ()))
+  qc (label "radix Word"        . prop_fullsort (R.sort :: Algo Word   ()))
+  qc (label "radix Int8"        . prop_fullsort (R.sort :: Algo Int8   ()))
+  qc (label "radix Int16"       . prop_fullsort (R.sort :: Algo Int16  ()))
+  qc (label "radix Int32"       . prop_fullsort (R.sort :: Algo Int32  ()))
+  qc (label "radix Int64"       . prop_fullsort (R.sort :: Algo Int64  ()))
+  qc (label "radix Int"         . prop_fullsort (R.sort :: Algo Int    ()))
+  qc (label "radix (Int, Int)"  . prop_fullsort (R.sort :: Algo (Int, Int) ()))
+
+  qc (label "flag Word8"       . prop_fullsort (AF.sort :: Algo Word8  ()))
+  qc (label "flag Word16"      . prop_fullsort (AF.sort :: Algo Word16 ()))
+  qc (label "flag Word32"      . prop_fullsort (AF.sort :: Algo Word32 ()))
+  qc (label "flag Word64"      . prop_fullsort (AF.sort :: Algo Word64 ()))
+  qc (label "flag Word"        . prop_fullsort (AF.sort :: Algo Word   ()))
+  qc (label "flag Int8"        . prop_fullsort (AF.sort :: Algo Int8   ()))
+  qc (label "flag Int16"       . prop_fullsort (AF.sort :: Algo Int16  ()))
+  qc (label "flag Int32"       . prop_fullsort (AF.sort :: Algo Int32  ()))
+  qc (label "flag Int64"       . prop_fullsort (AF.sort :: Algo Int64  ()))
+  qc (label "flag Int"         . prop_fullsort (AF.sort :: Algo Int    ()))
+  qc (label "flag ByteString"  . prop_fullsort (AF.sort :: Algo B.ByteString ()))
  where
  qc algo = quickCheckWith args algo
 
@@ -102,11 +117,11 @@ check_permutation = do
                                          (INT.partialSort :: SizeAlgo Int ())
   qc $ label "introselect"  . prop_sized (const . prop_permutation)
                                          (INT.select :: SizeAlgo Int ())
-  qc $ label "heapsort"     . prop_permutation (TH.sort :: Algo Int ())
+  qc $ label "heapsort"     . prop_permutation (H.sort :: Algo Int ())
   qc $ label "heappartial"  . prop_sized (const . prop_permutation)
-                                         (TH.partialSort :: SizeAlgo Int ())
+                                         (H.partialSort :: SizeAlgo Int ())
   qc $ label "heapselect"   . prop_sized (const . prop_permutation)
-                                         (TH.select :: SizeAlgo Int ())
+                                         (H.select :: SizeAlgo Int ())
   qc $ label "mergesort"    . prop_permutation (M.sort :: Algo Int    ())
   qc $ label "radix I8"     . prop_permutation (R.sort :: Algo Int8   ())
   qc $ label "radix I16"    . prop_permutation (R.sort :: Algo Int16  ())
@@ -118,9 +133,33 @@ check_permutation = do
   qc $ label "radix W32"    . prop_permutation (R.sort :: Algo Word32 ())
   qc $ label "radix W64"    . prop_permutation (R.sort :: Algo Word64 ())
   qc $ label "radix Word"   . prop_permutation (R.sort :: Algo Word   ())
+  qc $ label "flag I8"      . prop_permutation (AF.sort :: Algo Int8   ())
+  qc $ label "flag I16"     . prop_permutation (AF.sort :: Algo Int16  ())
+  qc $ label "flag I32"     . prop_permutation (AF.sort :: Algo Int32  ())
+  qc $ label "flag I64"     . prop_permutation (AF.sort :: Algo Int64  ())
+  qc $ label "flag Int"     . prop_permutation (AF.sort :: Algo Int    ())
+  qc $ label "flag W8"      . prop_permutation (AF.sort :: Algo Word8  ())
+  qc $ label "flag W16"     . prop_permutation (AF.sort :: Algo Word16 ())
+  qc $ label "flag W32"     . prop_permutation (AF.sort :: Algo Word32 ())
+  qc $ label "flag W64"     . prop_permutation (AF.sort :: Algo Word64 ())
+  qc $ label "flag Word"    . prop_permutation (AF.sort :: Algo Word   ())
+  qc $ label "flag ByteString" . prop_permutation (AF.sort :: Algo B.ByteString ())
  where
  qc prop = quickCheckWith args prop
 
+check_corners = do
+  qc "introsort empty"    $ prop_empty       (INT.sort        :: Algo Int ())
+  qc "intropartial empty" $ prop_sized_empty (INT.partialSort :: SizeAlgo Int ())
+  qc "introselect empty"  $ prop_sized_empty (INT.select      :: SizeAlgo Int ())
+  qc "heapsort empty"     $ prop_empty       (H.sort          :: Algo Int ())
+  qc "heappartial empty"  $ prop_sized_empty (H.partialSort   :: SizeAlgo Int ())
+  qc "heapselect empty"   $ prop_sized_empty (H.select        :: SizeAlgo Int ())
+  qc "mergesort empty"    $ prop_empty       (M.sort          :: Algo Int ())
+  qc "radixsort empty"    $ prop_empty       (R.sort          :: Algo Int ())
+  qc "flagsort empty"     $ prop_empty       (AF.sort         :: Algo Int ())
+ where
+ qc s prop = quickCheckWith (stdArgs { maxSuccess = 2 }) (label s prop)
+
 type BoundSAlgo e r = forall s mv. MVector mv e => mv s e -> e -> Int -> Int -> ST s r
 
 check_search_range = do
@@ -146,4 +185,6 @@ main = do putStrLn "Int tests:"
           putStrLn "Permutation:"
           check_permutation
           putStrLn "Search in range:"
-          check_search_range
\ No newline at end of file
+          check_search_range
+          putStrLn "Corner cases:"
+          check_corners
